@@ -10,7 +10,11 @@
 #' \code{process_folders} is used to process an entire directory. It searches each folder for
 #' PLEXOS solutions and applies \code{process_solution} to each one of them.
 #'
-#' @param folders Folder or folders to process (default is the working folder)
+#' The default folder is the working directory. If the wildcard \code{"*"} is provided, all
+#' the folders in the working directory will be processed (the list of folders if provided by
+#' the \code{\link{list_folders}} function).
+#'
+#' @param folders Folder(s) to process (See details)
 #' @param file Single PLEXOS solution file to process
 #' @param keep.temp Should temporary databases be preserved?
 #'
@@ -23,13 +27,20 @@
 process_folder <- function(folders = ".", keep.temp = FALSE) {
   # Check inputs
   assert_that(is.character(folders), is.flag(keep.temp))
+  
+  # Check for wildcard
+  if (length(folders) == 1) {
+    if (folders == "*") {
+      folders <- list_folders()
+    }
+  }
 
   # Check that folders exist
   are.dirs <- file.info(folders)$isdir
   are.dirs[is.na(are.dirs)] <- FALSE
   if(!all(are.dirs)) {
     not.dirs <- folders[!are.dirs]
-    stop(paste(not.dirs, collapse = ", "), " are not valid folders")
+    stop(paste(not.dirs, collapse = ", "), " are not valid folders", call. = FALSE)
   }
   
   # Iterate through list of folders
